@@ -1,6 +1,8 @@
 import os, string, sys, re
 import xml.dom.minidom
+import xml.parsers.expat
 from subprocess import Popen, PIPE
+from distutils import log
 
 try:
     # If we can import pyutil.version_class then use its regex.
@@ -67,41 +69,41 @@ def update(pkgname, verfilename, revision_number=False, loud=False, abort_if_sna
         cmd.append("--from-tag=^%s" % (pkgname,))
 
     try:
-        p = Popen(["darcs"] + cmd, stdout=PIPE, universal_newlines=True)
+        p = Popen(["darcs"] + cmd, stdout=PIPE, stderr=PIPE, universal_newlines=True)
     except OSError, ose:
         if ose.errno == 2 and '~' in os.environ['PATH']:
             expanded_path = os.environ['PATH'].replace('~', os.path.expanduser('~'))
             msg = ("WARNING: 'darcs' was not found. However '~' was found in your PATH. \n"
                    "Please note that bugs in python cause it to fail to traverse '~' in \n"
                    "the user's PATH.  Please fix your path, e.g. \nPATH=%s" )
-            print msg % (expanded_path,)
+            log.warn(msg % (expanded_path,))
         pass
     else:
-        output = p.communicate()[0]
+        (output, errput) = p.communicate()
         rc = p.returncode
     if rc != 0:
         try:
-            p = Popen(["realdarcs.exe"] + cmd, stdout=PIPE, universal_newlines=True)
+            p = Popen(["realdarcs.exe"] + cmd, stdout=PIPE, stderr=PIPE, universal_newlines=True)
         except OSError, ose:
             if ose.errno == 2 and '~' in os.environ['PATH']:
                 expanded_path = os.environ['PATH'].replace('~', os.path.expanduser('~'))
                 msg = ("WARNING: 'realdarcs.exe' was not found. However '~' was found in your PATH. \n"
                        "Please note that bugs in python cause it to fail to traverse '~' in \n"
                        "the user's PATH.  Please fix your path, e.g. \nPATH=%s" )
-                print msg % (expanded_path,)
+                log.warn(msg % (expanded_path,))
             pass
         else:
-            output = p.communicate()[0]
+            (output, errput) = p.communicate()
             rc = p.returncode
         if rc != 0:
+            if errput:
+                log.info("%s: darcs wrote to stderr: '%s'" % (EXE_NAME, errput,))
             if os.path.exists(verfilename):
-                if loud:
-                    print "%s: Failure from attempt to find version tags with 'darcs changes', and %s already exists, so leaving it alone." % (EXE_NAME, verfilename,)
+                log.info("%s: Failure from attempt to find version tags with 'darcs changes', and %s already exists, so leaving it alone." % (EXE_NAME, verfilename,))
                 return (0, read_version_py(verfilename))
             else:
-                if loud:
-                    print "%s: Failure from attempt to find version tags with 'darcs changes', and %s doesn't exist." % (EXE_NAME, verfilename,)
-                return (rc, None)
+                log.warn("%s: Failure from attempt to find version tags with 'darcs changes', and %s doesn't exist." % (EXE_NAME, verfilename))
+            return (rc, None)
 
     # Filter out bad chars that can cause the XML parser to give up in despair.
     # (Thanks to lelit of the tailor project and ndurner and warner for this hack.)
@@ -109,7 +111,14 @@ def update(pkgname, verfilename, revision_number=False, loud=False, abort_if_sna
     tt = string.maketrans(allbadchars, "-"*len(allbadchars))
     output = output.translate(tt)
 
-    doc = xml.dom.minidom.parseString(output)
+    # strip off trailing warning messages that darcs 2.3.1 writes to stdout
+    endi = output.find("</changelog>")+len("</changelog>")
+    output = output[:endi]
+    try:
+        doc = xml.dom.minidom.parseString(output)
+    except xml.parsers.expat.ExpatError, le:
+        le.args = tuple(le.args + (output,))
+        raise
 
     changelog = doc.getElementsByTagName("changelog")[0]
     patches = changelog.getElementsByTagName("patch")
@@ -139,13 +148,13 @@ def update(pkgname, verfilename, revision_number=False, loud=False, abort_if_sna
                 count_since_last_patch += 1
 
     if not last_tag:
+        if errput:
+            log.info("%s: darcs wrote to stderr: '%s'" % (EXE_NAME, errput,))
         if os.path.exists(verfilename):
-            if loud:
-                print "%s: I'm unable to find a tag in the darcs history matching \"%s\", so I'm leaving %s alone." % (EXE_NAME, regexstr, verfilename,)
+            log.warn("%s: I'm unable to find a tag in the darcs history matching \"%s\", so I'm leaving %s alone." % (EXE_NAME, regexstr, verfilename,))
             return (0, read_version_py(verfilename))
         else:
-            if loud:
-                print "%s: I'm unable to find a tag in the darcs history matching \"%s\", and %s doesn't exist." % (EXE_NAME, regexstr, verfilename,)
+            log.warn("%s: I'm unable to find a tag in the darcs history matching \"%s\", and %s doesn't exist." % (EXE_NAME, regexstr, verfilename,))
             return (0, None)
 
     if revision_number:
@@ -164,6 +173,5 @@ def update(pkgname, verfilename, revision_number=False, loud=False, abort_if_sna
             verstr = last_tag
 
     write_version_py(verstr, verfilename, EXE_NAME)
-    if loud:
-        print "%s: wrote '%s' into %s" % (EXE_NAME, verstr, verfilename,)
+    log.info("%s: wrote '%s' into %s" % (EXE_NAME, verstr, verfilename,))
     return (0, verstr)
